package jp.ne.sakura.sabowl.gpsnmeajp.intent_hce;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.cardemulation.HostNfcFService;
import android.os.Bundle;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;

/**
 * <br>Android OSからのHCE-F処理を受け取り，LocalBroadcastを用いてActivity側に受け渡すProxyサービス．
 * <br>また，Activity側からNFCリーダライタへの応答を受け取りシステムへ流す役目も持っている．
 *
 * <br>このServiceはシステム側から自動で呼び出されるため，Activityから明示的に起動する必要はない．
 * <br>ただしActivityにて，システム登録のためにenable()を行うこと
 * <br>Androidシステム側に登録するため，以下のことをすること．
 *
 * <br>1. manifest.xmlにserviceを登録．
 * <br>2. res/xml/lazyNfcFHCELiteProxySettings.xmlを作成
 * <br>3. res/values/strings.xmlにLazyNfcFHCELiteProxyServiceDescriptionを登録
 *
 <pre>
 LazyNfcFHCELite

 zlib License
 Copyright (c) 2018 GPS_NMEA_JP

 This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 3. This notice may not be removed or altered from any source distribution.
 </pre>
 *
 * @author gpsnmeajp
 * @version 1.0
 */
public class LazyNfcFHCELiteProxy extends HostNfcFService {
    /**
     * Service生成時(NFC通信開始時)に，Activity側からのsendResponsePacketを受信するためのLocalBroadcastManagerへの登録行います．
     */
    @Override
    public void onCreate() {
        super.onCreate();
        Log.d("HCEFService(NFCF)","onCreate");
        LocalBroadcastManager.getInstance(this).registerReceiver(mSendResponsePacketReceiver,
                new IntentFilter("LazyNfcFHCELite::sendResponsePacket"));
    }

    /**
     * Service破棄時(NFC通信終了時)に，LocalBroadcastManagerの登録解除を行います．
     */
    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.d("HCEFService(NFCF)","onDestroy");
        LocalBroadcastManager.getInstance(this).unregisterReceiver(mSendResponsePacketReceiver);
    }

    /**
     * システムから受信したNFCパケットをActivity側へLocalBroadcastを用いて転送します．
     * @param commandPacket システムから受信したNFCパケット
     * @param extras 追加情報 {@code null}
     * @return 常にnull (ここでは応答しない)
     */
    @Override
    public byte[] processNfcFPacket(byte[] commandPacket, Bundle extras)
    {
        Log.d("HCEFService(NFCF)","processNfcFPacket");

        //Forward
        Intent intent = new Intent("LazyNfcFHCELite::processNfcFPacket");
        intent.putExtra("packet", commandPacket);
        LocalBroadcastManager.getInstance(this).sendBroadcast(intent);
        return null;
    }

    /**
     * システムから受信したRF切断理由通知をActivity側へLocalBroadcastを用いて転送します．
     * @param reason RF切断理由
     */
    @Override
    public void onDeactivated(int reason) { //RF切断時
        Log.d("HCEFService(NFCF)","onDeactivated");

        //Forward
        Intent intent = new Intent("LazyNfcFHCELite::onDeactivated");
        intent.putExtra("reason", reason);
        LocalBroadcastManager.getInstance(this).sendBroadcast(intent);
    }

    private BroadcastReceiver mSendResponsePacketReceiver = new BroadcastReceiver() {
        /**
         * Activity側から転送された送信内容をシステムに渡し，NFC応答パケットとして送出します
         * @param context
         * @param intent
         */
        @Override
        public void onReceive(Context context, Intent intent) {
            byte responsePacket[] = intent.getByteArrayExtra("packet");
            sendResponsePacket(responsePacket);
            Log.d("HCEFService(NFCF)","sendResponsePacket");
        }
    };
}
